/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   QTreeView *m_view;

   struct TreeData {
      int id;
      QString name;
      int parentId;
   };

   QList<TreeData> getData();
};

MainWindow::MainWindow()
{
   setMinimumSize(400, 500);

   QLabel *label = new QLabel();
   label->setText("Tree View");
   label->setAlignment(Qt::AlignHCenter);

   QFont font = label->font();
   font.setPointSize(11);
   label->setFont(font);

   QList<MainWindow::TreeData> data = getData();

   QStandardItemModel *model = new QStandardItemModel();

   QStandardItem *rootItem = model->invisibleRootItem();
   QMap<int, QStandardItem *> parentMap;

   for (auto item : data) {
      if (item.parentId == 0) {
         QStandardItem *tmp = new QStandardItem;

         tmp->setText(item.name);
         tmp->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

         rootItem->appendRow(tmp);
         parentMap.insert(item.id, tmp);
      }
   }

   for (auto item : data) {
      if (item.parentId != 0) {
         QStandardItem *tmp = new QStandardItem;

         tmp->setText(item.name);
         tmp->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

         auto iter = parentMap.find(item.parentId);

         if (iter != parentMap.end()) {
            QStandardItem *parentItem = iter.value();

            if (parentItem != nullptr) {
               parentItem->appendRow(tmp);
            }
         }
      }
   }

   model->setHeaderData(0, Qt::Horizontal, QString("Recipe Categories"));
   model->sort(0);

   m_view = new QTreeView();
   m_view->setModel(model);
   m_view->expandAll();

   QPushButton *pb_close = new QPushButton();
   pb_close->setText("Close");

   QHBoxLayout *layout = new QHBoxLayout();
   layout->addStretch();
   layout->addWidget(pb_close);
   layout->addStretch();

   QVBoxLayout *layout_main = new QVBoxLayout(this);
   layout_main->setContentsMargins(20, 20, 20, 15);

   layout_main->addWidget(label);
   layout_main->addSpacing(10);
   layout_main->addWidget(m_view);
   layout_main->addSpacing(10);
   layout_main->addLayout(layout);

   connect(pb_close, &QPushButton::clicked, this, &QWidget::close);
}

QList<MainWindow::TreeData> MainWindow::getData()
{
   QList<MainWindow::TreeData> retval =
      {  {101, "Salad", 0},
         {102, "Bread", 0},
         {103, "KitchenSink Cookies", 105},
         {104, "Brownies", 105},
         {105, "Dessert", 0},
         {106, "Roasted Carrots", 114},
         {107, "Thai Curry", 108},
         {108, "Chicken", 0},
         {109, "Blueberry Muffins", 102},
         {110, "Chocolate Ice Cream", 105},
         {111, "Corn Bread", 102},
         {112, "Mixed Greens", 101},
         {113, "Soup", 0},
         {114, "Vegetables", 0},
      };

   return retval;
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
